﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Form1.cs" company="">
//
// </copyright>
// <summary>
//   The form 1.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace GridDragDropBoundCS
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Data;
    using System.Drawing;
    using System.Windows.Forms;

    using Telerik.WinControls;
    using Telerik.WinControls.UI;

    /// <summary>
    /// The form 1.
    /// </summary>
    public partial class Form1 : RadForm
    {
        #region Constants and Fields

        /// <summary>
        /// The rad grid view 1.
        /// </summary>
        private readonly DraggableGridView radGridView1;

        /// <summary>
        /// The rad grid view 2.
        /// </summary>
        private readonly DraggableGridView radGridView2;

        #endregion Constants and Fields

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            this.InitializeComponent();
            this.Text = "Drag Drop Bound mode example";
            this.Size = new Size(800, 600);
            this.radGridView1 = new DraggableGridView();
            radGridView1.AllowAddNewRow = false;
            radGridView1.Dock = DockStyle.Fill;
            this.radGridView2 = new DraggableGridView();
            radGridView2.AllowAddNewRow = false;
            radGridView2.Dock = DockStyle.Fill;

            var radGroupSource = new RadGroupBox();
            radGroupSource.HeaderText = "SourceGrid";

            var radGroupTarget = new RadGroupBox();
            radGroupTarget.HeaderText = "TargetGrid";

            this.radGridView1.AutoSizeColumnsMode = GridViewAutoSizeColumnsMode.Fill;
            radGroupSource.Controls.Add(this.radGridView1);

            this.radGridView2.AutoSizeColumnsMode = GridViewAutoSizeColumnsMode.Fill;
            radGroupTarget.Controls.Add(this.radGridView2);

            radGroupSource.Location = new Point(0, 0);
            radGroupSource.Size = new Size(this.ClientSize.Width / 2, this.ClientSize.Height);
            radGroupTarget.Location = new Point(this.ClientSize.Width / 2, 0);
            radGroupTarget.Size = new Size(this.ClientSize.Width / 2, this.ClientSize.Height);

            radGroupSource.Anchor = AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top | AnchorStyles.Bottom;
            radGroupTarget.Anchor = AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top | AnchorStyles.Bottom;

            var tableLayout = new TableLayoutPanel();
            tableLayout.Dock = DockStyle.Fill;
            tableLayout.RowStyles.Add(new RowStyle(SizeType.Percent, 100f));
            tableLayout.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 50f));
            tableLayout.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 50f));
            tableLayout.Controls.Add(radGroupSource, 0, 0);
            tableLayout.Controls.Add(radGroupTarget, 1, 0);
            this.Controls.Add(tableLayout);
        }

        #endregion Constructors and Destructors

        #region Methods

        /// <summary>
        /// The on load.
        /// </summary>
        /// <param name="e">
        /// The e.
        /// </param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            LoadDataSetDataSource();
            //LoadBusinessObjectsDataSource();
        }

        private void LoadBusinessObjectsDataSource()
        {
            this.Text += " - Bussiness Objects"; 
            this.radGridView1.DataSource = new TestsCollection(10, 1);
            this.radGridView2.DataSource = new TestsCollection(10, 10);
        }

        private void LoadDataSetDataSource()
        {
            this.Text += " - DataSet"; 
            this.radGridView1.DataSource = GetDataSet();
            this.radGridView1.DataMember = "Table1";
            this.radGridView2.DataSource = GetDataSet();
            this.radGridView2.DataMember = "Table1";
        }

        static DataSet GetDataSet()
        {
            var dataSet = new DataSet();
            //
            // Here we create a DataTable with four columns.
            //
            DataTable table = new DataTable();
            table.Columns.Add("Dosage", typeof(int));
            table.Columns.Add("Drug", typeof(string));
            table.Columns.Add("Patient", typeof(string));
            table.Columns.Add("Date", typeof(DateTime));

            //
            // Here we add five DataRows.
            //
            table.Rows.Add(25, "Indocin", "David", DateTime.Now);
            table.Rows.Add(50, "Enebrel", "Sam", DateTime.Now);
            table.Rows.Add(10, "Hydralazine", "Christoff", DateTime.Now);
            table.Rows.Add(21, "Combivent", "Janet", DateTime.Now);
            table.Rows.Add(100, "Dilantin", "Melanie", DateTime.Now);

            dataSet.Tables.Add(table);

            return dataSet;
        }

        #endregion Methods
    }

    /// <summary>
    /// The draggable grid view.
    /// </summary>
    public class DraggableGridView : RadGridView
    {
        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DraggableGridView"/> class.
        /// </summary>
        public DraggableGridView()
        {
            // comment this line to disable multi selection
            this.MultiSelect = true;

            var dragDropService = this.GridViewElement.GetService<RadGridViewDragDropService>();
            dragDropService.PreviewDragStart += this.dragDropService_PreviewDragStart;
            dragDropService.PreviewDragDrop += this.dragDropService_PreviewDragDrop;
            dragDropService.PreviewDragOver += this.dragDropService_PreviewDragOver;
            dragDropService.PreviewDragHint += this.dragDropService_PreviewDragHint;

            this.RowFormatting += this.DraggableGridView_RowFormatting;

            var gridBehavior = this.GridBehavior as BaseGridBehavior;
            gridBehavior.UnregisterBehavior(typeof(GridViewDataRowInfo));
            gridBehavior.RegisterBehavior(typeof(GridViewDataRowInfo), new MyGridDataRowBehavior());
        }

        #endregion Constructors and Destructors

        #region Properties

        /// <summary>
        /// Gets ThemeClassName.
        /// </summary>
        public override string ThemeClassName
        {
            get
            {
                return typeof(RadGridView).FullName;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// The draggable grid view_ row formatting.
        /// </summary>
        /// <param name="sender">
        /// The sender.
        /// </param>
        /// <param name="e">
        /// The e.
        /// </param>
        private void DraggableGridView_RowFormatting(object sender, RowFormattingEventArgs e)
        {
            e.RowElement.AllowDrag = true;
            e.RowElement.AllowDrop = true;
        }

        /// <summary>
        /// The get target row index.
        /// </summary>
        /// <param name="row">
        /// The row.
        /// </param>
        /// <param name="dropLocation">
        /// The drop location.
        /// </param>
        /// <returns>
        /// The get target row index.
        /// </returns>
        private int GetTargetRowIndex(GridDataRowElement row, Point dropLocation)
        {
            int halfHeight = row.Size.Height / 2;
            int index = row.RowInfo.Index;
            if (dropLocation.Y > halfHeight)
            {
                index++;
            }

            return index;
        }

        /// <summary>
        /// The move rows.
        /// </summary>
        /// <param name="targetGrid">
        /// The target grid.
        /// </param>
        /// <param name="dragGrid">
        /// The drag grid.
        /// </param>
        /// <param name="dragRows">
        /// The drag rows.
        /// </param>
        /// <param name="index">
        /// The index.
        /// </param>
        private void MoveRows(RadGridView targetGrid, RadGridView dragGrid, IList<GridViewRowInfo> dragRows, int index)
        {
            dragGrid.BeginUpdate();
            targetGrid.BeginUpdate();
            for (int i = dragRows.Count - 1; i >= 0; i--)
            {
                GridViewRowInfo row = dragRows[i];
                if (row is GridViewSummaryRowInfo)
                {
                    continue;
                }

                if (typeof(DataSet).IsAssignableFrom(targetGrid.DataSource.GetType()))
                {
                    // customize data set tables here if required
                    var sourceTable = ((DataSet)dragGrid.DataSource).Tables[0];
                    var targetTable = ((DataSet)targetGrid.DataSource).Tables[0];

                    var newRow = targetTable.NewRow();
                    foreach (GridViewCellInfo cell in row.Cells)
                    {
                        newRow[cell.ColumnInfo.Name] = cell.Value;
                    }

                    sourceTable.Rows.Remove(((DataRowView)row.DataBoundItem).Row);
                    targetTable.Rows.InsertAt(newRow, index);
                }
                else if (typeof(IList).IsAssignableFrom(targetGrid.DataSource.GetType()))
                {
                    var targetCollection = (IList)targetGrid.DataSource;
                    var sourceCollection = (IList)dragGrid.DataSource;

                    // handle custom removing of data bound item here if necessary
                    sourceCollection.Remove(row.DataBoundItem);

                    // format insert data here
                    targetCollection.Add(row.DataBoundItem);
                }
                else
                {
                    // handle custom data operations here
                }

                index++;
            }

            dragGrid.EndUpdate(true);
            targetGrid.EndUpdate(true);
        }

        /// <summary>
        /// The drag drop service_ preview drag drop.
        /// </summary>
        /// <param name="sender">
        /// The sender.
        /// </param>
        /// <param name="e">
        /// The e.
        /// </param>
        private void dragDropService_PreviewDragDrop(object sender, RadDropEventArgs e)
        {
            var rowElement = e.DragInstance as GridDataRowElement;
            if (rowElement == null)
            {
                return;
            }

            e.Handled = true;

            var dropTarget = e.HitTarget as RadItem;
            var targetGrid = dropTarget.ElementTree.Control as RadGridView;
            if (targetGrid == null)
            {
                return;
            }

            var dragGrid = rowElement.ElementTree.Control as RadGridView;

            if (targetGrid != dragGrid)
            {
                e.Handled = true;
                var dropTargetRow = dropTarget as GridDataRowElement;
                int index = dropTargetRow != null
                                ? this.GetTargetRowIndex(dropTargetRow, e.DropLocation)
                                : targetGrid.RowCount;

                this.MoveRows(targetGrid, dragGrid, dragGrid.SelectedRows, index);
            }
        }

        /// <summary>
        /// The drag drop service_ preview drag hint.
        /// </summary>
        /// <param name="sender">
        /// The sender.
        /// </param>
        /// <param name="e">
        /// The e.
        /// </param>
        private void dragDropService_PreviewDragHint(object sender, PreviewDragHintEventArgs e)
        {
            var dataRowElement = e.DragInstance as GridDataRowElement;
            if (dataRowElement != null && dataRowElement.ViewTemplate.MasterTemplate.SelectedRows.Count > 1)
            {
                // set custom drag hint for multiple rows here
                //e.DragHint = new Bitmap(this.imageList1.Images[6]);
                //e.UseDefaultHint = false;
            }
        }

        /// <summary>
        /// The drag drop service_ preview drag over.
        /// </summary>
        /// <param name="sender">
        /// The sender.
        /// </param>
        /// <param name="e">
        /// The e.
        /// </param>
        private void dragDropService_PreviewDragOver(object sender, RadDragOverEventArgs e)
        {
            if (e.DragInstance is GridDataRowElement)
            {
                e.CanDrop = e.HitTarget is GridDataRowElement || e.HitTarget is GridTableElement ||
                            e.HitTarget is GridSummaryRowElement;
            }
        }

        /// <summary>
        /// The drag drop service_ preview drag start.
        /// </summary>
        /// <param name="sender">
        /// The sender.
        /// </param>
        /// <param name="e">
        /// The e.
        /// </param>
        private void dragDropService_PreviewDragStart(object sender, PreviewDragStartEventArgs e)
        {
            e.CanStart = true;
        }

        #endregion Methods

        /// <summary>
        /// The my grid data row behavior.
        /// </summary>
        private class MyGridDataRowBehavior : GridDataRowBehavior
        {
            #region Methods

            /// <summary>
            /// The on mouse down left.
            /// </summary>
            /// <param name="e">
            /// The e.
            /// </param>
            /// <returns>
            /// The on mouse down left.
            /// </returns>
            protected override bool OnMouseDownLeft(MouseEventArgs e)
            {
                var dataRowElement = this.GetRowAtPoint(e.Location) as GridDataRowElement;

                if (dataRowElement != null)
                {
                    var dragDropService = this.GridViewElement.GetService<RadGridViewDragDropService>();
                    dragDropService.Start(dataRowElement);
                }

                return base.OnMouseDownLeft(e);
            }

            #endregion Methods
        }
    }

    /// <summary>
    /// The test.
    /// </summary>
    public class Test
    {
        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Test"/> class.
        /// </summary>
        /// <param name="id">
        /// The show all.
        /// </param>
        /// <param name="name">
        /// The show part.
        /// </param>
        public Test(int id, string name)
        {
            this.Id = id;
            this.Name = name;
        }

        #endregion Constructors and Destructors

        #region Properties

        /// <summary>
        /// Gets or sets ShowAll.
        /// </summary>
        public int Id
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets ShowPart.
        /// </summary>
        public string Name
        {
            get;
            set;
        }

        #endregion Properties
    }

    /// <summary>
    /// The tests collection.
    /// </summary>
    public class TestsCollection : BindingList<Test>
    {
        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TestsCollection"/> class.
        /// </summary>
        public TestsCollection(int noItems, int startIndex)
        {
            for (int i = startIndex; i <= noItems + startIndex; i++)
            {
                this.Add(new Test(i, "Name" + i));
            }
        }

        #endregion Constructors and Destructors
    }
}